@@ -9,6 +9,8 @@ end
 
 local SIZEOF_NUMBER = string.pack and #string.pack('n', 0.0) or 8
 local NUMBER_INTEGRAL = math.type and (math.type(0.0) == math.type(0)) or false
+local maxinteger
+local mininteger
 if not jit and _VERSION < 'Lua 5.3' then
     -- Lua 5.1 & 5.2
     local loadstring = loadstring or load
@@ -411,7 +413,7 @@ packers['float'] = function (buffer, n)
                                  sign, 0x00, 0x00, 0x00)
     else
         expo = expo + 0x7E
-        mant = (mant * 2.0 - 1.0) * ldexp(0.5, 24)
+        mant = floor((mant * 2.0 - 1.0) * ldexp(0.5, 24))
         buffer[#buffer+1] = char(0xCA,
                                  sign + floor(expo / 0x2),
                                  (expo % 0x2) * 0x80 + floor(mant / 0x10000),
@@ -430,7 +432,7 @@ packers['double'] = function (buffer, n)
     if mant ~= mant then
         buffer[#buffer+1] = char(0xCB,  -- nan
                                  0xFF, 0xF8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00)
-    elseif mant == huge then
+    elseif mant == huge or expo > 0x400 then
         if sign == 0 then
             buffer[#buffer+1] = char(0xCB,      -- inf
                                      0x7F, 0xF0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00)
@@ -438,12 +440,12 @@ packers['double'] = function (buffer, n)
             buffer[#buffer+1] = char(0xCB,      -- -inf
                                      0xFF, 0xF0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00)
         end
-    elseif mant == 0.0 and expo == 0 then
+    elseif (mant == 0.0 and expo == 0) or expo < -0x3FE then
         buffer[#buffer+1] = char(0xCB,  -- zero
                                  sign, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00)
     else
         expo = expo + 0x3FE
-        mant = (mant * 2.0 - 1.0) * ldexp(0.5, 53)
+        mant = floor((mant * 2.0 - 1.0) * ldexp(0.5, 53))
         buffer[#buffer+1] = char(0xCB,
                                  sign + floor(expo / 0x10),
                                  (expo % 0x10) * 0x10 + floor(mant / 0x1000000000000),
@@ -461,18 +463,18 @@ local set_number = function (number)
         packers['number'] = packers['signed']
     elseif number == 'float' then
         packers['number'] = function (buffer, n)
-            if floor(n) ~= n or n ~= n or n > 3.40282347e+38 or n < -3.40282347e+38 then
-                return packers['float'](buffer, n)
-            else
+            if floor(n) == n and n < maxinteger and n > mininteger then
                 return packers['integer'](buffer, n)
+            else
+                return packers['float'](buffer, n)
             end
         end
     elseif number == 'double' then
         packers['number'] = function (buffer, n)
-            if floor(n) ~= n or n ~= n or n == huge or n == -huge then
-                return packers['double'](buffer, n)
-            else
+            if floor(n) == n and n < maxinteger and n > mininteger then
                 return packers['integer'](buffer, n)
+            else
+                return packers['double'](buffer, n)
             end
         end
     else
@@ -594,7 +596,7 @@ local function unpack_map (c, n)
     for i = 1, n do
         local k = decode(c)
         local val = decode(c)
-        if k == nil then
+        if k == nil or k ~= k then
             k = m.sentinel
         end
         if k ~= nil then
@@ -864,7 +866,7 @@ unpackers['str32'] = function (c)
     c.i = i
     local n = ((b1 * 0x100 + b2) * 0x100 + b3) * 0x100 + b4
     local e = i+n-1
-    if e > j then
+    if e > j or n < 0 then
         c:underflow(e)
         s, i, j = c.s, c.i, c.j
         e = i+n-1
@@ -1026,7 +1028,7 @@ unpackers['ext32'] = function (c)
     i = i+1
     c.i = i
     local e = i+n-1
-    if e > j then
+    if e > j or n < 0 then
         c:underflow(e)
         s, i, j = c.s, c.i, c.j
         e = i+n-1
@@ -1105,21 +1107,28 @@ end
 set_string'string_compat'
 set_integer'unsigned'
 if NUMBER_INTEGRAL then
-    packers['double'] = packers['integer']
-    packers['float'] = packers['integer']
     set_number'integer'
 elseif SIZEOF_NUMBER == 4 then
-    packers['double'] = packers['float']
+    maxinteger = 16777215
+    mininteger = -maxinteger
     m.small_lua = true
+    unpackers['double'] = nil
+    unpackers['uint64'] = nil
+    unpackers['int64'] = nil
     set_number'float'
 else
+    maxinteger = 9007199254740991
+    mininteger = -maxinteger
     set_number'double'
+    if SIZEOF_NUMBER > 8 then
+        m.long_double = true
+    end
 end
 set_array'without_hole'
 
-m._VERSION = '0.3.3'
+m._VERSION = '0.3.6'
 m._DESCRIPTION = "lua-MessagePack : a pure Lua implementation"
-m._COPYRIGHT = "Copyright (c) 2012-2015 Francois Perrad"
+m._COPYRIGHT = "Copyright (c) 2012-2016 Francois Perrad"
 return m
 --
 -- This library is licensed under the terms of the MIT/X11 license,
