@@ -318,18 +318,18 @@ local set_number = function (number)
         packers['number'] = packers['signed']
     elseif number == 'float' then
         packers['number'] = function (buffer, n)
-            if math_type(n) == 'float' then
-                return packers['float'](buffer, n)
-            else
+            if math_type(n) == 'integer' then
                 return packers['integer'](buffer, n)
+            else
+                return packers['float'](buffer, n)
             end
         end
     elseif number == 'double' then
         packers['number'] = function (buffer, n)
-            if math_type(n) == 'float' then
-                return packers['double'](buffer, n)
-            else
+            if math_type(n) == 'integer' then
                 return packers['integer'](buffer, n)
+            else
+                return packers['double'](buffer, n)
             end
         end
     else
@@ -447,7 +447,7 @@ local function unpack_map (c, n)
     for i = 1, n do
         local k = decode(c)
         local val = decode(c)
-        if k == nil then
+        if k == nil or k ~= k then
             k = m.sentinel
         end
         if k ~= nil then
@@ -649,7 +649,7 @@ unpackers['str32'] = function (c)
     i = i+4
     c.i = i
     local e = i+n-1
-    if e > j then
+    if e > j or n < 0 then
         c:underflow(e)
         s, i, j = c.s, c.i, c.j
         e = i+n-1
@@ -809,7 +809,7 @@ unpackers['ext32'] = function (c)
     i = i+1
     c.i = i
     local e = i+n-1
-    if e > j then
+    if e > j or n < 0 then
         c:underflow(e)
         s, i, j = c.s, c.i, c.j
         e = i+n-1
@@ -891,16 +891,22 @@ if math_type(0.0) == math_type(0) then
     set_number'integer'
 elseif #pack('n', 0.0) == 4 then
     m.small_lua = true
+    unpackers['double'] = nil
+    unpackers['uint64'] = nil
+    unpackers['int64'] = nil
     set_number'float'
 else
     m.full64bits = true
     set_number'double'
+    if #pack('n', 0.0) > 8 then
+        m.long_double = true
+    end
 end
 set_array'without_hole'
 
-m._VERSION = '0.3.3'
+m._VERSION = '0.3.6'
 m._DESCRIPTION = "lua-MessagePack : a pure Lua implementation"
-m._COPYRIGHT = "Copyright (c) 2012-2015 Francois Perrad"
+m._COPYRIGHT = "Copyright (c) 2012-2016 Francois Perrad"
 return m
 --
 -- This library is licensed under the terms of the MIT/X11 license,
